@@ -1432,8 +1432,182 @@ class SbSession:
                 else:
                     print("Failed to unpublish file " + filename + " from public S3 bucket")
 
-    def start_spatial_service(self, item_id, filename):
-        """Creates a spatial service on a ScienceBase service definition (.sd) file in ArcGIS Online.
+    def delete_cloud_files(self, item_id, filenames):
+        """deletes a list of Cloud files on an item from the ScienceBase S3 content bucket and/or S3 publish bucket
+        and updates the item JSON accordingly
+
+        *can handle deletion of files from the S3 buckets to clean up the backend even if the item JSON is out of sync
+        (i.e. the files are not referenced in the item JSON)
+
+        :param item_id: The ID of the ScienceBase item
+        :param filenames: a list of filenames to be deleted
+        """
+        if not self._sbSessionEx.is_logged_in():
+            print(f'{self._username} not logged into Keycloak -- cloud services not available')
+        else:
+            item = self.get_item(item_id)
+
+            for filename in filenames:
+
+                cuid = ""
+                key = ""
+
+                if 'files' in item:
+                    for f in item['files']:
+                        if 'name' in f:
+                            if f['name'] == filename:
+                                if 'cuid' in f:
+                                    cuid = f['cuid']
+                                if 'key' in f:
+                                    key = f['key']
+                                break
+
+                if cuid == "":
+                    if 'facets' in item:
+                        for facet in item['facets']:
+                            if 'files' in facet:
+                                for f in facet['files']:
+                                    if f['name'] == filename:
+                                        if 'cuid' in f:
+                                            cuid = f['cuid']
+                                        if 'key' in f:
+                                            key = f['key']
+                                        break
+
+                # handle deletion of files from S3 buckets when the item JSON is out of sync
+                if cuid == "" and key == "":
+                    print("File " + filename + " not found on item")
+                    print("Will proceed to check for this file in the S3 content bucket and publish bucket and delete it from those locations if found.")
+
+                    key_val = item_id + "/" + filename
+
+                    params = {
+                        "key": key_val,
+                        "sb_env": self._env
+                    }
+
+                    if self._env == 'beta' or self._env == 'dev':
+                        delete_s3_file_url = "https://tqvcfyruhb.execute-api.us-west-2.amazonaws.com/prod/deleteS3Files"
+                    else:
+                        delete_s3_file_url = "https://ksrs49weqg.execute-api.us-west-2.amazonaws.com/prod/deleteS3Files"
+
+                    resp = self._session.post(delete_s3_file_url, json=params)
+
+                    print("Check completed.")
+
+                else:
+                    if cuid is None:
+                        cuid = ""
+
+                    if filename.endswith(".sd"):
+                        self.stop_esri_service(item_id, filename)
+
+                        #delete the .sd facet
+                        client_mutation_id = "delete_sd_facet"
+
+                        f = {}
+
+                        if 'facets' in item:
+                            facets = item['facets']
+                        else:
+                            facets = []
+
+                        for facet in facets:
+                            if facet['name'] == filename:
+                                f = facet
+                                break
+
+                        if f in facets:
+                            facets.remove(f)
+
+                        input = {
+                            "clientMutationId": client_mutation_id,
+                            "id": item_id,
+                            "itemPatch": {
+                                "facets": facets
+                            }
+                        }
+
+                        requests_session = requests.session()
+
+                        if self._env == 'beta' or self._env == 'dev':
+                            graphql_url = "https://api-beta.staging.sciencebase.gov/graphql"
+                        else:
+                            graphql_url = "https://api.sciencebase.gov/graphql"
+
+                        token = self._sbSessionEx._token
+
+                        headers = {"Content-Type": "application/json",
+                                   "Accept": "application/json",
+                                   "Authorization": "Bearer " + token
+                                   }
+
+                        query = """ mutation UpdateItem($input: UpdateItemInput!) {
+                                            updateItem(input: $input) {
+                                                item {
+                                                    distributionLinks {
+                                                        uri
+                                                    }
+                                                }
+                                            }
+                                          }
+                                        """
+
+                        failed_retry_time = 10
+                        for tries in range(3):
+                            try:
+                                resp = requests_session.post(
+                                    graphql_url,
+                                    headers=headers,
+                                    json={'query': query, 'variables': {'input': input}}
+                                )
+                                break
+                            except requests.exceptions.Timeout as e:
+                                print("DeleteSdFacet Timeout: " + repr(e))
+                            except requests.exceptions.RequestException as e:
+                                print("DeleteSdFacet RequestException: " + repr(e))
+                            # Wait before we try again
+                            time.sleep(failed_retry_time)
+                            failed_retry_time *= 2
+
+                        print("RESPONSE (GRAPHQL): " + resp.text)
+
+                        if "SyntaxError" in resp.text:
+                            print("SyntaxError: " + resp.text)
+                        else:
+                            print("DeleteSdFacet Completed Successfully")
+
+                    input = {"cuid": cuid, "key": key}
+
+                    response = self._sbSessionEx.delete_cloud_file(input)
+                    print(response)
+
+                    # handle deletion of on-premise files published to public bucket
+                    if cuid == "":
+                        key_val = item_id + "/" + filename
+
+                        params = {
+                            "key": key_val,
+                            "sb_env": self._env
+                        }
+
+                        if self._env == 'beta' or self._env == 'dev':
+                            delete_s3_file_url = "https://tqvcfyruhb.execute-api.us-west-2.amazonaws.com/prod/deleteS3Files"
+                        else:
+                            delete_s3_file_url = "https://ksrs49weqg.execute-api.us-west-2.amazonaws.com/prod/deleteS3Files"
+
+                        resp = self._session.post(delete_s3_file_url, json=params)
+
+                    if 'errors' in response:
+                        print("Failed to delete file " + filename)
+                        print("Please try again.")
+                        if 'INTERNAL_SERVER_ERROR' in response:
+                            print("Internal server error. Please check to make sure this file is a cloud file.")
+                    else:
+                        print("Successfully deleted " + filename + " from ScienceBase item and associated S3 bucket(s)")
+
+    def start_esri_service(self, item_id, filename):
+        """Creates a spatial service on a ScienceBase service definition (.sd) file in ArcGIS Online or ArcGIS Server.
         The service definition file must have been published to the public ScienceBase S3 bucket.
         User will receive an email notification when process is complete.
                :param item_id: The ID of the ScienceBase item
@@ -1476,17 +1650,19 @@ class SbSession:
                     "type": "single"
                 }
 
-                start_spatial_service_url = "https://rwxatj0usl.execute-api.us-west-2.amazonaws.com/prod/startSpatialService"
-
+                if self._env == 'beta' or self._env == 'dev':
+                    start_spatial_service_url = "https://gggcfbu5gh.execute-api.us-west-2.amazonaws.com/prod/startSpatialService"
+                else:
+                    start_spatial_service_url = "https://rwxatj0usl.execute-api.us-west-2.amazonaws.com/prod/startSpatialService"
                 self._session.post(start_spatial_service_url, json=params)
 
                 print("Triggered spatial service creation in ArcGIS Online.")
                 return True
 
-    def stop_spatial_service(self, item_id, filename):
-        """Stops a spatial service that had been published on a ScienceBase service definition (.sd) file in ArcGIS Online.
+    def stop_esri_service(self, item_id, filename):
+        """Stops a spatial service that had been published on a ScienceBase service definition (.sd) file in ArcGIS Online or ArcGIS Server.
                :param item_id: The ID of the ScienceBase item
-               :param filename: The filename of the .sd file in ScienceBase on which the ArcGIS Online spatial service had been published
+               :param filename: The filename of the .sd file in ScienceBase on which the ArcGIS Online or ArcGIS Server spatial service had been published
         """
         if not self.is_logged_in():
             print("Please log in and retry.")
@@ -1497,30 +1673,35 @@ class SbSession:
 
                 if 'facets' in item:
                     for facet in item['facets']:
-                        if 'files' in facet:
-                            for f in facet['files']:
-                                if f['name'] == filename:
-                                    if facet['serverType'] == 'AGOL_Feature_Server' or facet['serverType'] == 'AGOL_WMTS_Server':
-                                        if 'enabledServices' in facet:
-                                            if len(facet['enabledServices']) == 2:
-                                                agol_id_1 = facet['enabledServices'][0]
-                                                agol_id_2 = facet['enabledServices'][1]
-                                                file_path_used = facet['filePathUsed']
-                                                params = {
-                                                    "filename": file_path_used,
-                                                    "item_id": item_id,
-                                                    "agol_id_1": agol_id_1,
-                                                    "agol_id_2": agol_id_2,
-                                                    "email": self._username
-                                                }
+                        if facet['name'] == filename:
+                            if 'serverType' in facet:
+                                if facet['serverType'] == 'AGOL_Feature_Server' or facet['serverType'] == 'AGOL_WMTS_Server':
+                                    if 'enabledServices' in facet:
+                                        if len(facet['enabledServices']) == 2:
+                                            agol_id_1 = facet['enabledServices'][0]
+                                            agol_id_2 = facet['enabledServices'][1]
+                                            file_path_used = facet['filePathUsed']
+                                            params = {
+                                                "filename": file_path_used,
+                                                "item_id": item_id,
+                                                "agol_id_1": agol_id_1,
+                                                "agol_id_2": agol_id_2,
+                                                "email": self._username
+                                            }
+                                            if self._env == 'beta' or self._env == 'dev':
+                                                stop_spatial_service_url = "https://02j686fjyf.execute-api.us-west-2.amazonaws.com/prod/stopSpatialService"
+                                            else:
                                                 stop_spatial_service_url = "https://qk9hqzs5yf.execute-api.us-west-2.amazonaws.com/prod/stopSpatialService"
-                                                self._session.post(stop_spatial_service_url, json=params)
-                                                print("Triggered deletion of spatial service in ArcGIS Online.")
-                                                return True
-
-                                    elif facet['servicePath'] != '' and facet['serviceId'] != '' and facet['processingState'] == 'success':
+                                            self._session.post(stop_spatial_service_url, json=params)
+                                            print("Triggered deletion of spatial service in ArcGIS Online.")
+                                            return True
+                                elif 'servicePath' in facet and 'serviceId' in facet and 'processingState' in facet:
+                                    if facet['servicePath'] != '' and facet['serviceId'] != '' and facet['processingState'] == 'success':
                                         payload = {'operation': 'delete'}
-                                        url = "https://www.sciencebase.gov/catalog/item/createProcessJob/" + item_id
+                                        if self._env == 'beta' or self._env == 'dev':
+                                            url = "https://beta.sciencebase.gov/catalog/item/createProcessJob/" + item_id
+                                        else:
+                                            url = "https://www.sciencebase.gov/catalog/item/createProcessJob/" + item_id
                                         self._session.get(url, params=payload)
                                         print("Triggered deletion of spatial service from ScienceBase ArcGIS Server instance.")
                                         return True
